@@ -23,7 +23,6 @@ except ImportError:  # pragma: no cover
 from strawberry.custom_scalar import ScalarDefinition
 from strawberry.enum import EnumDefinition
 from strawberry.lazy_type import LazyType
-from strawberry.scalars import SCALAR_TYPES
 from strawberry.type import (
     StrawberryList,
     StrawberryOptional,
@@ -31,6 +30,7 @@ from strawberry.type import (
     StrawberryTypeVar,
 )
 from strawberry.types.types import TypeDefinition
+from strawberry.unset import _Unset
 from strawberry.utils.typing import is_generic, is_type_var
 
 
@@ -83,8 +83,6 @@ class StrawberryAnnotation:
             return self.create_list(evaled_type)
         elif self._is_optional(evaled_type):
             return self.create_optional(evaled_type)
-        elif self._is_scalar(evaled_type):
-            return evaled_type
         elif self._is_union(evaled_type):
             return self.create_union(evaled_type)
         elif is_type_var(evaled_type):
@@ -116,7 +114,10 @@ class StrawberryAnnotation:
     def create_optional(self, evaled_type: Any) -> StrawberryOptional:
         types = evaled_type.__args__
         non_optional_types = tuple(
-            filter(lambda x: x is not type(None), types)  # noqa: E721
+            filter(
+                lambda x: x is not type(None) and x is not _Unset,  # noqa: E721
+                types,
+            )
         )
 
         # Note that passing a single type to `Union` is equivalent to not using `Union`
@@ -200,15 +201,6 @@ class StrawberryAnnotation:
 
         return annotation_origin == list
 
-    @classmethod
-    def _is_scalar(cls, annotation: Any) -> bool:
-        type_ = getattr(annotation, "__supertype__", annotation)
-
-        if type_ in SCALAR_TYPES:
-            return True
-
-        return hasattr(annotation, "_scalar_definition")
-
     @classmethod
     def _is_strawberry_type(cls, evaled_type: Any) -> bool:
         # Prevent import cycles
