@@ -1,10 +1,18 @@
 import dataclasses
-from typing import List, Optional, Type, cast
+import inspect
+import types
+from typing import Callable, List, Optional, Sequence, Type, TypeVar, cast, overload
 
-from .exceptions import MissingFieldAnnotationError, MissingReturnAnnotationError
+from strawberry.schema_directive import StrawberrySchemaDirective
+
+from .exceptions import (
+    MissingFieldAnnotationError,
+    MissingReturnAnnotationError,
+    ObjectIsNotClassError,
+)
 from .field import StrawberryField, field
 from .types.type_resolver import _get_fields
-from .types.types import FederationTypeParams, TypeDefinition
+from .types.types import TypeDefinition
 from .utils.str_converters import to_camel_case
 from .utils.typing import __dataclass_transform__
 
@@ -92,12 +100,14 @@ def _process_type(
     is_input: bool = False,
     is_interface: bool = False,
     description: Optional[str] = None,
-    federation: Optional[FederationTypeParams] = None,
+    directives: Optional[Sequence[StrawberrySchemaDirective]] = (),
+    extend: bool = False,
 ):
     name = name or to_camel_case(cls.__name__)
 
     interfaces = _get_interfaces(cls)
     fields = _get_fields(cls)
+    is_type_of = getattr(cls, "is_type_of", None)
 
     cls._type_definition = TypeDefinition(
         name=name,
@@ -105,32 +115,78 @@ def _process_type(
         is_interface=is_interface,
         interfaces=interfaces,
         description=description,
-        federation=federation or FederationTypeParams(),
+        directives=directives,
         origin=cls,
+        extend=extend,
         _fields=fields,
+        is_type_of=is_type_of,
     )
 
     # dataclasses removes attributes from the class here:
     # https://github.com/python/cpython/blob/577d7c4e/Lib/dataclasses.py#L873-L880
     # so we need to restore them, this will change in future, but for now this
     # solution should suffice
-
     for field_ in fields:
         if field_.base_resolver and field_.python_name:
-            setattr(cls, field_.python_name, field_.base_resolver.wrapped_func)
+            wrapped_func = field_.base_resolver.wrapped_func
+
+            # Bind the functions to the class object. This is necessary because when
+            # the @strawberry.field decorator is used on @staticmethod/@classmethods,
+            # we get the raw staticmethod/classmethod objects before class evaluation
+            # binds them to the class. We need to do this manually.
+            if isinstance(wrapped_func, staticmethod):
+                bound_method = wrapped_func.__get__(cls)
+                field_.base_resolver.wrapped_func = bound_method
+            elif isinstance(wrapped_func, classmethod):
+                bound_method = types.MethodType(wrapped_func.__func__, cls)
+                field_.base_resolver.wrapped_func = bound_method
+
+            setattr(cls, field_.python_name, wrapped_func)
 
     return cls
 
 
+T = TypeVar("T")
+
+
+@overload
 @__dataclass_transform__(order_default=True, field_descriptors=(field, StrawberryField))
 def type(
-    cls: Type = None,
+    cls: T,
     *,
     name: str = None,
     is_input: bool = False,
     is_interface: bool = False,
     description: str = None,
-    federation: Optional[FederationTypeParams] = None,
+    directives: Optional[Sequence[StrawberrySchemaDirective]] = (),
+    extend: bool = False,
+) -> T:
+    ...
+
+
+@overload
+@__dataclass_transform__(order_default=True, field_descriptors=(field, StrawberryField))
+def type(
+    *,
+    name: str = None,
+    is_input: bool = False,
+    is_interface: bool = False,
+    description: str = None,
+    directives: Optional[Sequence[StrawberrySchemaDirective]] = (),
+    extend: bool = False,
+) -> Callable[[T], T]:
+    ...
+
+
+def type(
+    cls=None,
+    *,
+    name=None,
+    is_input=False,
+    is_interface=False,
+    description=None,
+    directives=(),
+    extend=False,
 ):
     """Annotates a class as a GraphQL type.
 
@@ -142,15 +198,24 @@ def type(
     """
 
     def wrap(cls):
-        wrapped = _wrap_dataclass(cls)
+        if not inspect.isclass(cls):
+            if is_input:
+                exc = ObjectIsNotClassError.input
+            elif is_interface:
+                exc = ObjectIsNotClassError.interface
+            else:
+                exc = ObjectIsNotClassError.type
+            raise exc(cls)
 
+        wrapped = _wrap_dataclass(cls)
         return _process_type(
             wrapped,
             name=name,
             is_input=is_input,
             is_interface=is_interface,
             description=description,
-            federation=federation,
+            directives=directives,
+            extend=extend,
         )
 
     if cls is None:
@@ -159,13 +224,35 @@ def type(
     return wrap(cls)
 
 
+@overload
 @__dataclass_transform__(order_default=True, field_descriptors=(field, StrawberryField))
 def input(
-    cls: Type = None,
+    cls: T,
     *,
     name: str = None,
     description: str = None,
-    federation: Optional[FederationTypeParams] = None,
+    directives: Optional[Sequence[StrawberrySchemaDirective]] = (),
+) -> T:
+    ...
+
+
+@overload
+@__dataclass_transform__(order_default=True, field_descriptors=(field, StrawberryField))
+def input(
+    *,
+    name: str = None,
+    description: str = None,
+    directives: Optional[Sequence[StrawberrySchemaDirective]] = (),
+) -> Callable[[T], T]:
+    ...
+
+
+def input(
+    cls=None,
+    *,
+    name=None,
+    description=None,
+    directives=(),
 ):
     """Annotates a class as a GraphQL Input type.
     Example usage:
@@ -175,7 +262,7 @@ def input(
     """
 
     return type(
-        cls, name=name, description=description, federation=federation, is_input=True
+        cls, name=name, description=description, directives=directives, is_input=True
     )
 
 
@@ -185,7 +272,7 @@ def interface(
     *,
     name: str = None,
     description: str = None,
-    federation: Optional[FederationTypeParams] = None,
+    directives: Optional[Sequence[StrawberrySchemaDirective]] = (),
 ):
     """Annotates a class as a GraphQL Interface.
     Example usage:
@@ -198,13 +285,12 @@ def interface(
         cls,
         name=name,
         description=description,
-        federation=federation,
+        directives=directives,
         is_interface=True,
     )
 
 
 __all__ = [
-    "FederationTypeParams",
     "TypeDefinition",
     "input",
     "interface",
