@@ -1,5 +1,5 @@
 from itertools import chain
-from typing import Optional, cast
+from typing import Dict, Optional, cast
 
 from graphql.type import is_object_type, is_specified_directive
 from graphql.utilities.print_schema import (
@@ -15,27 +15,45 @@ from graphql.utilities.print_schema import (
 )
 
 from strawberry.field import StrawberryField
+from strawberry.schema_directive import Location, StrawberrySchemaDirective
 from strawberry.types.types import TypeDefinition
 
 from .schema import BaseSchema
 
 
-def print_federation_field_directive(field: Optional[StrawberryField]) -> str:
+def print_schema_directive_params(params: Dict) -> str:
+    if not params:
+        return ""
+
+    return "(" + ", ".join(f'{name}: "{value}"' for name, value in params.items()) + ")"
+
+
+def print_schema_directive(
+    directive: StrawberrySchemaDirective, schema: BaseSchema
+) -> str:
+    params = directive.instance.__dict__ if directive.instance else {}
+
+    directive_name = schema.config.name_converter.from_directive(directive)
+
+    return f" @{directive_name}{print_schema_directive_params(params)}"
+
+
+def print_field_directives(field: Optional[StrawberryField], schema: BaseSchema) -> str:
     if not field:
         return ""
 
-    out = ""
+    directives = (
+        directive
+        for directive in field.directives
+        if any(
+            location in [Location.FIELD_DEFINITION, Location.INPUT_FIELD_DEFINITION]
+            for location in directive.locations
+        )
+    )
 
-    if field.federation.provides:
-        out += f' @provides(fields: "{" ".join(field.federation.provides)}")'
-
-    if field.federation.requires:
-        out += f' @requires(fields: "{" ".join(field.federation.requires)}")'
-
-    if field.federation.external:
-        out += " @external"
-
-    return out
+    return "".join(
+        (print_schema_directive(directive, schema=schema) for directive in directives)
+    )
 
 
 def print_fields(type_, schema: BaseSchema) -> str:
@@ -57,39 +75,37 @@ def print_fields(type_, schema: BaseSchema) -> str:
             + f"  {name}"
             + print_args(field.args, "  ")
             + f": {field.type}"
-            + print_federation_field_directive(strawberry_field)
+            + print_field_directives(strawberry_field, schema=schema)
             + print_deprecated(field.deprecation_reason)
         )
 
     return print_block(fields)
 
 
-def print_federation_key_directive(type_, schema: BaseSchema):
+def print_extends(type_, schema: BaseSchema):
+    strawberry_type = cast(TypeDefinition, schema.get_type_by_name(type_.name))
+
+    if strawberry_type and strawberry_type.extend:
+        return "extend "
+
+    return ""
+
+
+def print_type_directives(type_, schema: BaseSchema) -> str:
     strawberry_type = cast(TypeDefinition, schema.get_type_by_name(type_.name))
 
     if not strawberry_type:
         return ""
 
-    keys = strawberry_type.federation.keys
+    directives = (
+        directive
+        for directive in strawberry_type.directives or []
+        if any(location in [Location.OBJECT] for location in directive.locations)
+    )
 
-    parts = []
-
-    for key in keys:
-        parts.append(f'@key(fields: "{key}")')
-
-    if not parts:
-        return ""
-
-    return " " + " ".join(parts)
-
-
-def print_extends(type_, schema: BaseSchema):
-    strawberry_type = cast(TypeDefinition, schema.get_type_by_name(type_.name))
-
-    if strawberry_type and strawberry_type.federation.extend:
-        return "extend "
-
-    return ""
+    return "".join(
+        (print_schema_directive(directive, schema=schema) for directive in directives)
+    )
 
 
 def _print_object(type_, schema: BaseSchema) -> str:
@@ -98,7 +114,7 @@ def _print_object(type_, schema: BaseSchema) -> str:
         + print_extends(type_, schema)
         + f"type {type_.name}"
         + print_implemented_interfaces(type_)
-        + print_federation_key_directive(type_, schema)
+        + print_type_directives(type_, schema)
         + print_fields(type_, schema)
     )
 
